#!/usr/bin/python

import sys
import re
import pprint
from rpmUtils import miscutils as rpm_misc
from copy import deepcopy


log = open(sys.argv[1], 'r')

interest = []
pending_builds = {}
re_accepted_builds = re.compile(r"^.*Accepted builds:")
re_pending_builds = re.compile(r"^.*Pending builds: \[(?P<pending_list>.*)\]")
re_problematic_builds = re.compile(r"^.*Problematic builds:")

line = True # do not change!

# accepted builds
while line:
    line = log.readline()
    if re_accepted_builds.search(line) is not None:
        interest.append(line)
        break

# get pending builds
while line:
    line = log.readline()
    l = re_pending_builds.search(line)
    if l is not None:
        interest.append(line)
        # string "['foo', 'bar']" on input
        # split using ',' as delimiter -> list of strings ["'foo'", " 'bar'"]
        # strip whitespace & leading/trailing ' -> list of strings ["foo", "bar"]
        pending_builds = map(lambda s: s.strip()[1:-1], l.groupdict()['pending_list'].split(','))
        break

# problematic builds
while line:
    line = log.readline()
    if re_problematic_builds.search(line) is not None:
        interest.append(line)
        break

interest.append("\n")

# create dict with pending builds as keys, None as values {"foo":None, "bar":None}
p_builds = dict().fromkeys(pending_builds)

# for each builds, add dictionary storing interesting data
for key in p_builds:
    p_builds[key] = {'interest': deepcopy(interest),
                     'depsolving': [],
                     'rpms': [], 
                     're': [
                              re.compile(re.escape(key)),
                              re.compile('TSINFO: Current Transaction'),
                              re.compile('Dependency Process ending'),
                              re.compile('Running transaction check'),
                              re.compile('Packages skipped because of dependency problems'),
                           ],
                     'pretty': ""}

re_fetching = re.compile(r"^.*Fetching RPMs for (?P<build>.*)\s*")
re_rpm = re.compile(r"^.*\s(?P<rpm>.*rpm).*")
re_running_depcheck = re.compile(r"^.*Running './depcheck.*")

# get 'fetching packages' parts
while line:
    line = log.readline()
    if re_running_depcheck.search(line) is not None:
        break
    l = re_fetching.search(line)
    if l is not None:
        build = l.groupdict()['build']
        if build not in pending_builds:
            continue

        p_builds[build]['interest'].append(line)
        while line:
            line = log.readline()
            if line.lower().strip().endswith('.rpm'):
                p_builds[build]['interest'].append(line)

                rpm = re_rpm.search(line).groupdict()['rpm']
                p_builds[build]['rpms'].append(rpm)

                (n,v,r,e,a) = rpm_misc.splitFilename(rpm)
                p_builds[build]['re'].append(re.compile(re.escape(n)))
            else:
                p_builds[build]['interest'].append("\n")
                break

# checking packages against repo
re_repo_check = re.compile(r'Checking packages against repo')
while line:
    line = log.readline()
    if re_repo_check.search(line) is not None:
        for build in p_builds:
            p_builds[build]['interest'].append(line)
            p_builds[build]['interest'].append("\n")
        break

# skip non-interesting part
re_pkgsack = re.compile(r'Setting up Package Sacks')
while line:
    line = log.readline()
    if re_pkgsack.search(line) is not None:
        break

re_depsolving_problems = re.compile("has depsolving problems")
re_arrow = re.compile(re.escape("-->"))

# get the rest of lines containing build or rpm name
while line:
    line = log.readline()
    # First of all, we need to check for the
    # 'XYZ has depsolving problems' line
    # If this line is found, this line, and all subsequend lines
    # starting with '-->' are stored in a buffer (log_lines).
    #
    # Because 'depsolving problems' can contain rpms from
    # multiple builds and/or other rpms, it's crucial
    # to scan all the lines in the 'depsolving problem'
    # buffer, and append the buffer to each affected pending build's
    # interesting lines.
    #
    # Because the inner while loop reads 'one more line' (i.e.
    # the first line not starting with '-->'). This line need's
    # to be completely re-examined, beacause it can contain
    # not only 'other information', but also next 'depsolving problems'
    # block. 
    # To achieve this effect, it's required to seek back in the file
    # and enforce 'jump' to the beginning of the outer loop. Hence the
    # try-catch block and `raise StopIteration` line.
    try:
        # Start of 'depsolving problems' block found
        if re_depsolving_problems.search(line) is not None:
            # store the first line to buffer
            log_lines = [line]
            affected_builds = []

            # as long, as the subsequent lines start with '-->'
            # add them to the buffer.
            line = log.readline()
            while re_arrow.search(line) is not None:
                log_lines.append(line)
                line = log.readline()

            # Seek back to the beginning of the lastly read line.
            # This line is not a part of the 'depsolving problems'
            # block, and needs to be completely re-examined
            log.seek(log.tell() - len(line))

            # determine which builds are affected by this depsolving
            # problems block ...
            for l in log_lines:
                for build in p_builds:
                    for regexp in p_builds[build]['re']:
                        if regexp.search(l) is not None:
                            affected_builds.append(build)
                            break

            # ... and store it to the interesting lines
            # of each affected build
            affected_builds = set(affected_builds)
            for build in affected_builds:
                p_builds[build]['depsolving'].append(log_lines)
                p_builds[build]['interest'].extend(log_lines)

            raise StopIteration()

        # simply check whether the line contains
        # build or rpm-from-the-build name
        for build in p_builds:
            for regexp in p_builds[build]['re']:
                if regexp.search(line) is not None:
                    p_builds[build]['interest'].append(line)
                    break

    except StopIteration:
        pass


log.close()

re_strip_begining = re.compile("^.*?\|.*?\|( \[stdout\])* ")
re_skip_looking_for = re.compile("looking for.*as a requirement of.*")
re_skip_already_ts = re.compile("Package.*already in ts")
re_skip_already_updated = re.compile("Not Updating Package that is already updated")
#re_skip_
re_tsinfo = re.compile("TSINFO: Current Transaction")
re_dependency_ending = re.compile('Dependency Process ending')
re_running_transaction = re.compile('Running transaction check')

for build in p_builds:
    for line in p_builds[build]['interest']:
        if (
                (re_skip_looking_for.search(line) is not None) or
                (re_skip_already_ts.search(line) is not None) or
                (re_skip_already_updated.search(line) is not None)
           ):
            continue

        l = re_strip_begining.sub('', line)

        if (
                (re_depsolving_problems.search(line) is not None) or
                (re_tsinfo.search(line) is not None) or
                (re_running_transaction.search(line) is not None)
           ):
            p_builds[build]['pretty'] += "\n"


        p_builds[build]['pretty'] += l

        if (re_dependency_ending.search(line) is not None):
            p_builds[build]['pretty'] += "\n"



    print "================ %s ===================\n" % build
    print p_builds[build]['pretty']
    print "=====================================\n"



